from __future__ import division, absolute_import, print_function

__all__ = ['ArgumentError', 'F', 'beta', 'binomial', 'chi_square',
           'exponential', 'gamma', 'get_seed', 'multinomial',
           'multivariate_normal', 'negative_binomial', 'noncentral_F',
           'noncentral_chi_square', 'normal', 'permutation', 'poisson',
           'randint', 'random', 'random_integers', 'standard_normal',
           'uniform', 'seed']

from numpy.oldnumeric.random_array import *
